// ============================================================================
// Module Name: IPat_GfClkMux
// Revision: 20171031a
// Author: Altmo toolbox
// Description:
//  + Glitch-free clock multiplexer
// ============================================================================

`timescale 1ps/100fs

module IPat_GfClkMux (
    clkin0,
    clkin1,
    clkout,
    sel,
    rst_n
);

// local parameter ============================================================
localparam LP_UD = 1;	// unit delay to avoid racing

// port definition ============================================================
input   clkin0;	// clock input
input   clkin1;	// clock input
output  clkout;	// clock output
input   sel;	// clock enable
input   rst_n;	// asynchronous reset


// wire and register ----------------------------------------------------------
// reset synchronizer
wire            w_rst_n_sync0;
wire            w_rst_n_sync1;
// clock gate
wire    [1:0]   w_lt_clken0;
wire    [1:0]   w_lt_clken1;
reg             r_lt_clken0;
reg             r_lt_clken1;
// enable synchronizer
wire    [1:0]   i_sync0;
wire    [1:0]   o_sync0;
wire    [1:0]   i_sync1;
wire    [1:0]   o_sync1;
// each clock output
wire            w_clkout0;
wire            w_clkout1;

// clock output ---------------------------------------------------------------
assign clkout = w_clkout0 | w_clkout1; // or output from all clock gate

// clock gate -----------------------------------------------------------------
always @(negedge clkin0 or negedge w_rst_n_sync0) begin
    if (w_rst_n_sync0 == 1'b0) begin
        r_lt_clken0 <= #LP_UD 1'b0;
    end
    else begin
        r_lt_clken0 <= #LP_UD w_lt_clken0;
    end
end

always @(negedge clkin1 or negedge w_rst_n_sync1) begin
    if (w_rst_n_sync1 == 1'b0) begin
        r_lt_clken1 <= #LP_UD 1'b0;
    end
    else begin
        r_lt_clken1 <= #LP_UD w_lt_clken1;
    end
end

assign w_clkout0 = clkin0 & r_lt_clken0;  // clock gate output
assign w_clkout1 = clkin1 & r_lt_clken1;  // clock gate output

// clock gate input -----------------------------------------------------------
assign w_lt_clken0 = ~o_sync0[1] &  o_sync0[0];
assign w_lt_clken1 =  o_sync1[1] & ~o_sync1[0];

// enable synchronizer --------------------------------------------------------
assign i_sync0 = { r_lt_clken1,        ~sel };
assign i_sync1 = {         sel, r_lt_clken0 };

IPat_SyncVar #(
    .P_IPAT_SYNCVAR_DWIDTH(2),
    .P_IPAT_SYNCVAR_PDEPTH(2)
) clken_sync0 (
    .clk(clkin0),
    .i_sync(i_sync0),
    .o_sync(o_sync0),
    .rst_n(w_rst_n_sync0)
);

IPat_SyncVar #(
    .P_IPAT_SYNCVAR_DWIDTH(2),
    .P_IPAT_SYNCVAR_PDEPTH(2)
) clken_sync1 (
    .clk(clkin1),
    .i_sync(i_sync1),
    .o_sync(o_sync1),
    .rst_n(w_rst_n_sync1)
);

// reset synchronizer ---------------------------------------------------------
IPat_SyncRst rst_n_sync0 (
    .clk(clkin0),
    .rst_n(rst_n),
    .rst_n_sync(w_rst_n_sync0)
);

IPat_SyncRst rst_n_sync1 (
    .clk(clkin1),
    .rst_n(rst_n),
    .rst_n_sync(w_rst_n_sync1)
);


endmodule
